<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Service\Action\BaseActionService;
use App\Service\Dto\Classes\ChangeProfileDto;
use App\Service\Dto\DtoServiceInterface;
use App\Service\Response\ResponseServiceInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\SecurityBundle\Security;

class ChangeProfile extends BaseActionService
{
    private ?User $user;

    public function __construct(
        private ResponseServiceInterface $profileResponse,
        private DtoServiceInterface      $profileDto,
        private ManagerRegistry          $doctrine,
        Security                         $security
    )
    {
        $this->user = $security->getUser();
        parent::__construct($profileResponse);
    }

    public function runAction(): void
    {
        /** @var ChangeProfileDto $dto */
        $dto = $this->profileDto->getClass();

        /** @var ?User $userExists */
        $userExists = $this->doctrine->getRepository(User::class)
            ->findOneByUniq($dto->email, $dto->phoneNumber, $this->user->getId());

        if ($userExists !== null) {
            if ($userExists->getEmail() === $dto->email) {
                $this->profileResponse->addError('Email занят другим пользователем');
            } elseif ($userExists->getPhoneNumber() === $dto->phoneNumber) {
                $this->profileResponse->addError('Номер телефона занят другим пользователем');
            } else {
                $this->profileResponse->addError('Email или номер телефона занят другим пользователем');
            }
        } else {
            $changed = false;

            if ($dto->name !== null && $dto->name !== $this->user->getName()) {
                $this->user->setName($dto->name);
                $changed = true;
            }

            if ($dto->surname !== null && $dto->surname !== $this->user->getSurname()) {
                $this->user->setSurname($dto->surname);
                $changed = true;
            }

            if ($dto->patronymic !== null && $dto->patronymic !== $this->user->getPatronymic()) {
                $this->user->setPatronymic($dto->patronymic);
                $changed = true;
            }

            if ($dto->phoneNumber !== null && $dto->phoneNumber !== $this->user->getPhoneNumber()) {
                $this->user->setPhoneNumber($dto->phoneNumber);
                $changed = true;
            }

            if ($dto->email !== null  && $dto->email !== $this->user->getEmail()) {
                $this->user->setEmail($dto->email);
                $changed = true;
            }

            if ($changed) {
                try {
                    $em = $this->doctrine->getManager();
                    $em->persist($this->user);
                    $em->flush();
                    $this->profileResponse->setData($this->user);
                } catch (\Exception $e) {
                    $this->profileResponse->addError('Ошибка сохранения профиля');
                }
            } else {
                $this->profileResponse->setData($this->user);
            }
        }
    }

    public function validate(): bool
    {
        if ($this->user === null) {
            $this->profileResponse->addError('Вы не авторизованы');
            return false;
        }
        return $this->profileDto->validate($this->profileResponse);
    }
}