<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Service\Action\BaseActionService;
use App\Service\Dto\Classes\RecoveryCodeDto;
use App\Service\Dto\DtoServiceInterface;
use App\Service\Response\ResponseServiceInterface;
use Doctrine\Persistence\ManagerRegistry;

class CheckRecoveryCode extends BaseActionService
{
    public function __construct(
        private DtoServiceInterface      $recoveryCodeDto,
        private ResponseServiceInterface $response,
        private ManagerRegistry          $doctrine,
    )
    {
        parent::__construct($response);
    }

    /**
     * Восстановление учетной записи по коду
     *
     * @return void
     */
    public function runAction(): void
    {
        /** @var RecoveryCodeDto $dto */
        $dto = $this->recoveryCodeDto->getClass();
        /** @var User $userExists */
        $userExists = $this->doctrine->getRepository(User::class)
            ->findOneByUniq($dto->email, $dto->phoneNumber);

        if ($userExists) {
            $currentDate = new \DateTime();
            $code = $dto->code;
            $registerCode = $userExists->getRegisterCode();
            if ($registerCode === null) {
                $this->response->addError('Код подтверждения не отправлен');
            } else {
                if ($registerCodeDate = $registerCode->getDate()) {
                    if ($registerCode->getCode() === $code && $currentDate->getTimestamp() < $registerCodeDate->getTimestamp()) {
                        try {
                            $userExists->setDeleted(false);
                            $em = $this->doctrine->getManager();
                            $em->persist($userExists);
                            $em->remove($registerCode);
                            $em->flush();
                            $this->response->addMessage('Профиль восстановлен');
                        } catch (\Exception $exception) {
                            $this->response->addError('Ошибка восстановления профиля');
                        }
                    } else {
                        $this->response->addError('Код недействителен');
                    }
                } else {
                    $this->response->addError('Код недействителен');
                }
            }
        } else {
            $this->response->addError('Пользователь не найден');
        }
    }

    public function validate(): bool
    {
        return $this->recoveryCodeDto->validate($this->response);
    }
}