<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Entity\UserHistory;
use App\Service\Action\BaseActionService;
use App\Service\Response\ResponseServiceInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\SecurityBundle\Security;

class DeleteImage extends BaseActionService
{
    private ?User $user;

    public function __construct(
        private ResponseServiceInterface $response,
        private ManagerRegistry          $doctrine,
        Security                         $security
    )
    {
        $this->user = $security->getUser();
        parent::__construct($response);
    }

    public function runAction(): void
    {
        $image = $this->user->getImage();
        if ($image) {
            $em = $this->doctrine->getManager();

            try {
                $newUserHistory = new UserHistory();
                $newUserHistory->setType(UserHistory::TYPE_DELETE);
                $newUserHistory->setField('image');
                $newUserHistory->setValue($image->getName());
                $this->user->addUserHistory($newUserHistory);
                $em->remove($image);
                $em->flush();
                $this->response->addMessage('Изображение удалено');
            } catch (\Exception $exception) {
                $this->response->addError('Ошибка удаления изображения');
            }
        } else {
            $this->response->addError('Нет изображения');
        }
    }

    public function validate(): bool
    {
        if ($this->user === null) {
            $this->response->addError('Вы не авторизованы');
            return false;
        }
        if ($this->user->isDeleted()) {
            $this->response->addError('Профиль удален');
            return false;
        }
        return true;
    }
}