<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Service\Action\BaseActionService;
use App\Service\Response\ResponseServiceInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\SecurityBundle\Security;

class ResetEmail extends BaseActionService
{
    private ?User $user;

    /**
     * @param ResponseServiceInterface $response
     * @param ManagerRegistry $doctrine
     * @param Security $security
     */
    public function __construct(
        private ResponseServiceInterface $response,
        private ManagerRegistry          $doctrine,
        Security                         $security
    )
    {
        $this->user = $security->getUser();
        parent::__construct($response);
    }

    public function runAction(): void
    {
        if ($email = $this->user->getLastConfirmEmail()) {
            if ($email === $this->user->getEmail()) {
                $this->response->addMessage('Подтвержденный email уже установлен');
            } else {
                try {
                    $this->user->setEmail($email);
                    $em = $this->doctrine->getManager();
                    $em->persist($this->user);
                    $em->flush();
                    $this->response->addMessage('Установлен email: ' . $email);
                } catch (\Exception $e) {
                    $this->response->addError('Ошибка сохранения email');
                }
            }
        } else {
            $this->response->addError('Нет последнего подтвержденного email');
        }
    }

    public function validate(): bool
    {
        if ($this->user === null) {
            $this->response->addError('Вы не авторизованы');
            return false;
        }
        if ($this->user->isDeleted()) {
            $this->response->addError('Профиль удален');
            return false;
        }
        return true;
    }
}