<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Service\Action\BaseActionService;
use App\Service\Dto\Classes\RecoveryDto;
use App\Service\Dto\DtoServiceInterface;
use App\Service\Response\ResponseServiceInterface;
use App\Service\Send\SendServiceInterface;
use Doctrine\Persistence\ManagerRegistry;

class SendResetPasswordCode extends BaseActionService
{
    public function __construct(
        private ResponseServiceInterface $response,
        private DtoServiceInterface      $recoveryDto,
        private ManagerRegistry          $doctrine,
        private SendServiceInterface     $passwordCodeSendService
    )
    {
        parent::__construct($response);
    }

    public function runAction(): void
    {
        /** @var RecoveryDto $dto */
        $dto = $this->recoveryDto->getClass();
        /** @var User $userExists */
        $userExists = $this->doctrine->getRepository(User::class)
            ->findOneByUniq($dto->email, $dto->phoneNumber);

        if ($userExists !== null) {
            $this->passwordCodeSendService->setUser($userExists);
            $this->passwordCodeSendService->setResponse($this->response);
            $this->passwordCodeSendService->send();
        } else {
            $this->response->addError('Пользователь не найден');
        }
    }

    public function validate(): bool
    {
        return $this->recoveryDto->validate($this->response);
    }
}